#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 480
#define TILE_SIZE 40
#define ENTITY_COUNT 100

#include <SDL.h>
#include <SDL_image.h>
#include <SDL_mixer.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <stdlib.h>

#include "utils.h"
#include "pix_font_meta.h"

#define BLOOD_RED 0xCF,0x00,0x00
#define SKY_BLUE 0x00,0xFF,0xFF
#define GOLDEN 0xFF,0xFF,0x00

typedef enum {FACE_UP=3,FACE_DOWN=0,FACE_LEFT=1,FACE_RIGHT=2} face_t;

typedef struct {
	float x,y;//where is it
	uint16_t type;//what it is
	face_t facing;//what way its facing (down=0,left,right,UP)
	uint8_t frame;//frame that you are in (used for timed frame changes to know whats next)
	uint16_t flags;//assorted, bit0 is moving, bit1 is will drop item when killed\
		bit2 is ignores terrain, bit3 is no entity collisions(bit2 and 3 non functonal)
	float speed;
	float xTarg,yTarg;//target for pathing
	int health;
	int dmg;
} entity;

typedef struct {
	int xOff,yOff;
	SDL_Rect hitBox;
	float optDist;
	float circleBox;
} entityTypeData;

typedef struct {
	uint16_t floorID;//what the ground is made of
	uint16_t objectID;//what the rest is made of
	uint8_t flags;//flags about the tile, bit0 is if the tile is walkable
} tile;

#include "map.h"
int mode=0;//0=menu, 1=game
int submode=0;//goes with mode many meanings [0,0] is font menu, [1,x]is in game [0,1]is game over, [0,2]is game win
int level=0;
int playerID=0;
tile map[MAP_WIDTH][MAP_HEIGHT];

int running = 1;
spriteSheet playerS;
spriteSheet floorS;
spriteSheet objectS;
spriteSheet mobS;
spriteSheet itemS;
SDL_Texture* hud;
SDL_Texture* splash;

Mix_Music* gMusic;
Mix_Chunk* sfx[7];

bitFont testFont;
char text[80];
char text2[80];
char text3[80];

entity Entities[ENTITY_COUNT];//Entities[playerID] is the player
int entityC=3;//how many entities are on the list
entityTypeData entDat[]={{16,48,{-11,-15,22,15},0,11},{16,48,{-11,-22,22,22},35,11},{16,48,{-9,-18,18,18},15,11},
	{16,32,{-5,-10,10,10},0,5},{17,29,{-13,-24,24,24},0,12},{16,26,{-8,-16,16,16},0,8},{17,29,{-13,-24,24,24},0,12},
	{17,23,{-7,-14,14,14},0,10},{16,31,{-12,-24,24,24},0,12}};
//player,ghost,bat,thrown weapon,dry cloth, bomb, wet cloth,gem,key

int inventory;//id of the thing in your inventory
//4=dry cloth,5=bomb,6=wet cloth,7=gem, 8=key
int dropCD=0;

uint8_t* curKeys;
time_t time2,time3,gameStart,gameTime,frameTime;
//game time is used for global animations, others are used for assorted things


//tamp variables
SDL_Rect tempRectC,tempRectD;

int oninit();
int loadMedia();
void onEvent(SDL_Event* e);
void handleKeys();
void onTick();
inline void moveE(float distX, float distY, entity* e);
void tryMoveE(float distX, float distY, entity* e);
tile* getTile(float x, float y);
int isValid(float x, float y);
void onRender();
void onClose();
void reset(int lv);

int main(int argc, char* argv[]){
	srand(clock());
	reset(0);
	if(onInit()<=0)return -1;
	time3=clock();
	gameStart=clock();
	while(running){
		while(SDL_PollEvent(&Event)){
			onEvent(&Event);
		}
		if(mode==1){
			onTick();
		}
		onRender();
	}
	onClose();
	return 0;
}

int onInit(){
	debuglog = fopen("debuglog.txt","wt");
	if( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0 )
	{
		fprintf(debuglog, "SDL could not initialize! SDL_Error: %s\n", SDL_GetError());
		return 0;
	}
	//Set texture filtering to off
	if( !SDL_SetHint( SDL_HINT_RENDER_SCALE_QUALITY, "0" ) ){
		fprintf(debuglog, "Warning: texture filtering not off!" );
	}
	//Create window
	window = SDL_CreateWindow( "Jam-crawl v1.0 by Terra Barber", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_SHOWN );
	if( window == NULL ){
		fprintf(debuglog, "Window could not be created! SDL_Error: %s\n", SDL_GetError() );
		return 0;
	}
	gRenderer = SDL_CreateRenderer( window, -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC );
	if( gRenderer == NULL ){
		fprintf(debuglog, "Renderer could not be created! SDL Error: %s\n", SDL_GetError() );
		return 0;
	}	
	//Initialize renderer color
	SDL_SetRenderDrawColor( gRenderer, 0x00, 0x00, 0x00, 0xFF );
	//init image loading
	int imgFlags = IMG_INIT_PNG;
	if( !( IMG_Init( imgFlags ) & imgFlags )){
		fprintf(debuglog, "SDL_image could not initialize! SDL_image Error: %s\n", IMG_GetError() );
		return 0;
	}
	 //Initialize SDL_mixer
	 if(Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 2048)<0){
		printf("SDL_mixer could not initialize! SDL_mixer Error: %s\n", Mix_GetError() );
		return 0;
	}
	//load content
	return loadMedia();
}

int loadMedia(){
	//graphics
	if(SDLP_loadSheet("assets/char1.png", &playerS, 32, 48))fprintf(debuglog, "Loaded player sprite sheet:%d\n",(playerS.rows)*(playerS.cols));
	if(SDLP_loadSheet("assets/floors.png", &floorS, 40, 40))fprintf(debuglog, "Loaded floor sprite sheet:%d\n",(floorS.rows)*(floorS.cols));
	if(SDLP_loadSheet("assets/objects.png", &objectS, 40, 120))fprintf(debuglog, "Loaded object sprite sheet:%d\n",(objectS.rows)*(objectS.cols));
	if(SDLP_loadSheet("assets/mobs.png", &mobS, 32, 48))fprintf(debuglog, "Loaded mob sprite sheet:%d\n",(mobS.rows)*(mobS.cols));
	if(SDLP_loadSheet("assets/items.png", &itemS, 32, 32))fprintf(debuglog, "Loaded item sprite sheet:%d\n",(itemS.rows)*(itemS.cols));
	if(SDLP_loadFont("assets/pix_font.png", &testFont, '!', '~', 9, 3, pixFontMeta))fprintf(debuglog, "Loaded Font\n");
	if((hud=SDLP_loadTex("assets/hud.png"))!=NULL)fprintf(debuglog, "Loaded HUD\n");
	if((splash=SDLP_loadTex("assets/BG.png"))!=NULL)fprintf(debuglog, "Loaded BG\n");
	//sound
	if((gMusic = Mix_LoadMUS("assets/the_door.ogg"))!=NULL){fprintf(debuglog, "Loaded music\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if((sfx[0] = Mix_LoadWAV("assets/pickup.wav"))!=NULL){fprintf(debuglog, "Loaded sfx1\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if((sfx[1] = Mix_LoadWAV("assets/bomb.wav"))!=NULL){fprintf(debuglog, "Loaded sfx2\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if((sfx[2] = Mix_LoadWAV("assets/sizzle.wav"))!=NULL){fprintf(debuglog, "Loaded sfx3\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if((sfx[3] = Mix_LoadWAV("assets/slam.wav"))!=NULL){fprintf(debuglog, "Loaded sfx4\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if((sfx[4] = Mix_LoadWAV("assets/hurt.wav"))!=NULL){fprintf(debuglog, "Loaded sfx5\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if((sfx[5] = Mix_LoadWAV("assets/hit.wav"))!=NULL){fprintf(debuglog, "Loaded sfx6\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if((sfx[6] = Mix_LoadWAV("assets/splash.wav"))!=NULL){fprintf(debuglog, "Loaded sfx6\n");}else{fprintf(debuglog, "Failed to load music! SDL_mixer Error: %s\n", Mix_GetError());}
	if(Mix_PlayMusic(gMusic, -1)==-1) {fprintf(debuglog, "Failed to play music! SDL_mixer Error: %s\n", Mix_GetError());}
	return 1;
}

void onEvent(SDL_Event* e){
	if(e->type == SDL_QUIT){
		running=0;
		return;
	}
	if(e->type == SDL_KEYDOWN){
		switch(e->key.keysym.sym){
			case SDLK_SPACE:
				if(mode==1){
					if(inventory!=0){//use inventory item
						int tx,ty;
						switch(Entities[playerID].facing){
							case FACE_DOWN:
								tx=Entities[playerID].x;
								ty=Entities[playerID].y+25;
								break;
							case FACE_UP:
								tx=Entities[playerID].x;
								ty=Entities[playerID].y-25;
								break;
							case FACE_LEFT:
								tx=Entities[playerID].x-25;
								ty=Entities[playerID].y+1;
								break;
							case FACE_RIGHT:
								tx=Entities[playerID].x+25;
								ty=Entities[playerID].y+1;
								break;
							default:
								tx=Entities[playerID].x;
								ty=Entities[playerID].y+1;
								break;
						}
						tile* targT = getTile(tx+10,ty);
						tile* targTB;
						if((inventory==4)&&(targT->floorID>=42)&&(targT->floorID<=47)){
							inventory=6;
							Mix_PlayChannel(-1, sfx[6], 0);
							break;
						}
						if((inventory==6)&&(targT->floorID>=36)&&(targT->floorID<=41)){
							targT->floorID +=12;
							targT->flags = 0;
							inventory=0;
							Mix_PlayChannel(-1, sfx[2], 0);
							break;
						}
						if((inventory==5)&&(targT->objectID>=1)&&(targT->objectID<=5)){
							targT->objectID = 0;
							targT->flags = 0;
							inventory=0;
							Mix_PlayChannel(-1, sfx[1], 0);
							break;
						}
						if((inventory==8)&&(targT->objectID>=36)&&(targT->objectID<=37)){
							if(targT->objectID==36){
								targTB = getTile(tx+10+TILE_SIZE,ty);
							}else{
								targTB = getTile(tx+10-TILE_SIZE,ty);
							}
							targT->objectID +=2;
							targT->flags &= ~_BV(0);
							targTB->objectID +=2;
							targTB->flags &= ~_BV(0);
							inventory=0;
							Mix_PlayChannel(-1, sfx[3], 0);
							break;
						}
						if(targT->flags&_BV(0))break;
						Entities[entityC]=(entity){tx,ty,inventory,0,0,0,0,tx,ty,0,0};
						entityC++;
						inventory=0;
						dropCD=gameTime+1000;
					}else{//throw weapon
						int tx,ty;
						switch(Entities[playerID].facing){
							case FACE_DOWN:
								tx=Entities[playerID].x;
								ty=Entities[playerID].y+1000;
								break;
							case FACE_UP:
								tx=Entities[playerID].x;
								ty=Entities[playerID].y-1000;
								break;
							case FACE_LEFT:
								tx=Entities[playerID].x-1000;
								ty=Entities[playerID].y+1;
								break;
							case FACE_RIGHT:
								tx=Entities[playerID].x+1000;
								ty=Entities[playerID].y+1;
								break;
							default:
								tx=Entities[playerID].x;
								ty=Entities[playerID].y;
								break;
						}
						Entities[entityC]=(entity){Entities[playerID].x,Entities[playerID].y+1,3,0,0,1,200,tx,ty,gameTime+1000,0};
						entityC++;
					}
				}else if(mode==0){
					if(e->key.keysym.sym==SDLK_SPACE){//start new game
						mode=1;
						submode=0;
						reset(0);
					}
				}
				break;
			case SDLK_DELETE:
				reset(level);
				break;
			default:
				break;
		}
	}
}

void handleKeys(){
	curKeys = SDL_GetKeyboardState(NULL);
	Entities[playerID].flags&=~_BV(0);//set not moving by default
	Entities[playerID].xTarg=Entities[playerID].x;
	Entities[playerID].yTarg=Entities[playerID].y;
	if(curKeys[SDL_SCANCODE_D]){
		Entities[playerID].facing=FACE_RIGHT;
		Entities[playerID].xTarg=Entities[playerID].x+100;
		Entities[playerID].flags|=_BV(0);
	}else if(curKeys[SDL_SCANCODE_A]){
		Entities[playerID].facing=FACE_LEFT;
		Entities[playerID].xTarg=Entities[playerID].x-100;
		Entities[playerID].flags|=_BV(0);
	}
	if(curKeys[SDL_SCANCODE_W]){
		Entities[playerID].facing=FACE_UP;
	Entities[playerID].yTarg=Entities[playerID].y-100;
		Entities[playerID].flags|=_BV(0);
	}else if(curKeys[SDL_SCANCODE_S]){
		Entities[playerID].facing=FACE_DOWN;
	Entities[playerID].yTarg=Entities[playerID].y+100;
		Entities[playerID].flags|=_BV(0);
	}
	if(!(Entities[playerID].flags&_BV(0))){
		Entities[playerID].frame=0;
	}
}

void onTick(){
	time2 = clock();
	gameTime = time2-gameStart;
	frameTime = time2-time3;
	handleKeys();
	doAI();
	if(Entities[playerID].health<=0){//you are dead
		mode=0;submode=1;//menu with you are dead screen
	}
	if(Entities[playerID].flags&_BV(0)){//if player is moving
		Entities[playerID].frame = (gameTime/130)%4;
	}
	int i;
	for(i=0;i<entityC;i++){
		if(Entities[i].flags&_BV(0)){//try movement with collision detection
			tryMoveE(Entities[i].xTarg-Entities[i].x, Entities[i].yTarg-Entities[i].y, Entities+i);
		}
	}
	time3=time2;
	if(inventory==7){//if we have the gem
		if(level<MAX_LEVEL){
			reset(level+1);
		}else{
			mode=0; 
			submode=2;
		}
	}
	sortEnt();
	camPos=(SDL_Point){Entities[playerID].x-(SCREEN_WIDTH/2),Entities[playerID].y-(SCREEN_HEIGHT/2)};
}

void doAI(){
	int i,j;
	for(i=0;i<entityC;i++){
		//loop through entities doing ai
		switch(Entities[i].type){
			case 0://no ai(player)
				break;
			case 1://ghost
				if(((gameTime%3000)<500)&&(((gameTime-frameTime)%3000)>500)){//once every 3000ms
					Entities[i].xTarg=((float)rand()/RAND_MAX)*(MAP_WIDTH*TILE_SIZE);
					Entities[i].yTarg=((float)rand()/RAND_MAX)*(MAP_HEIGHT*TILE_SIZE);
					calcFace(Entities+i);
					Entities[i].flags |= _BV(0);
				}
				if(Entities[i].health<=0){
					if(Entities[i].flags&_BV(1)){
						Entities[i].type=4;
						Entities[i].flags=0;
						Entities[i].xTarg=Entities[i].x;
						Entities[i].yTarg=Entities[i].y;
					}else{
						EDelete(i);
						i--;
						goto endAI;
					}
				}
				break;
			case 2://bat
				if((Entities[i].xTarg!=Entities[playerID].x)||(Entities[i].yTarg!=Entities[playerID].y)){
					Entities[i].xTarg=Entities[playerID].x;
					Entities[i].yTarg=Entities[playerID].y;
					calcFace(Entities+i);
					Entities[i].flags |= _BV(0);
				}
				if(Entities[i].health<=0){
					if(Entities[i].flags&_BV(1)){
						Entities[i].type=5;
						Entities[i].flags=0;
						Entities[i].xTarg=Entities[i].x;
						Entities[i].yTarg=Entities[i].y;
					}else{
						EDelete(i);
						i--;
						goto endAI;
					}
				}
				break;
			case 3://bullet
				for(j=1;j<entityC;j++){
					if(i==j)continue;
					if(j==playerID)continue;
					if(distF(Entities[i].x,Entities[i].y,Entities[j].x,Entities[j].y)<(entDat[Entities[i].type].circleBox+entDat[Entities[j].type].circleBox)){
						Entities[j].health-=Entities[playerID].dmg;
						EDelete(i);
						i--;
						Mix_PlayChannel(-1, sfx[5], 0);
						goto endAI;
					}
				}
				if(gameTime>Entities[i].health){//destory this(time out)
					EDelete(i);
					i--;
					goto endAI;
				}
				break;
			case 4://cloth
			case 5://bomb
			case 6://wet cloth
			case 7://gem
			case 8://key
				if(inventory==0){//is inventory empty
					if(distF(Entities[i].x,Entities[i].y,Entities[playerID].x,Entities[playerID].y)<(entDat[Entities[i].type].circleBox+entDat[Entities[playerID].type].circleBox)){
						if(dropCD<gameTime){
							inventory=Entities[i].type;
							Mix_PlayChannel(-1, sfx[0], 0);
							EDelete(i);
							i--;
							goto endAI;
						}
					}
				}
				break;
			default:
				break;
		}
		if(distF(Entities[i].xTarg,Entities[i].yTarg,Entities[i].x,Entities[i].y)<entDat[Entities[i].type].optDist){
			Entities[i].flags&=~_BV(0);
		}
		if((distF(Entities[i].x,Entities[i].y,Entities[playerID].x,Entities[playerID].y)<entDat[Entities[i].type].optDist)&&(i!=playerID)){
			if(((gameTime%500)<100)&&(((gameTime-frameTime)%500)>100)){//once every 500ms
				Entities[playerID].health-=Entities[i].dmg;
				Mix_PlayChannel(-1, sfx[4], 0);
			}
		}
		endAI:
		i=i;
	}
}

void EDelete(int i){
	int j;
	for(j=(i+1);j<entityC;j++){
		if(j==playerID)playerID--;
		Entities[j-1]=Entities[j];
	}
	entityC--;
}

void calcFace(entity* e){
	if(distF(0,e->yTarg,0,e->y)>distF(0,e->xTarg,0,e->x)){//moveing mainly up/down
		e->facing=((e->yTarg-e->y)>0)?FACE_DOWN:FACE_UP;
	}else{//moving mainly left or right
		e->facing=((e->xTarg-e->x)>0)?FACE_RIGHT:FACE_LEFT;
	}
}

inline void moveE(float distX, float distY, entity* e){//force move of entity
	e->x += distX;
	e->y += distY;
}

void tryMoveE(float distX, float distY, entity* e){//attempt moment with appropriate collision detection
	//limit move distance
	if(((distX*distX)+(distY*distY))>((e->speed*frameTime/CLOCKS_PER_SEC)*(e->speed*frameTime/CLOCKS_PER_SEC))){
		float scale = (float)(e->speed*frameTime/CLOCKS_PER_SEC)/sqrt((distX*distX)+(distY*distY));
		distX *= scale;
		distY *= scale;
	}
	//make temp entity to test move
	entity tempE = *e;
	moveE(distX,distY,&tempE);
	if((tempE.x+entDat[tempE.type].hitBox.x) < 0)distX=0;//left map on left
	if((tempE.y+entDat[tempE.type].hitBox.y) < 0)distY=0;//left map on top
	if((tempE.x+entDat[tempE.type].hitBox.x+entDat[tempE.type].hitBox.w) > MAP_WIDTH*TILE_SIZE)distX=0;//left map on right
	if((tempE.y+entDat[tempE.type].hitBox.y+entDat[tempE.type].hitBox.h) > MAP_HEIGHT*TILE_SIZE)distY=0;//left map on bottom
	if((e->flags)&_BV(2)){
		moveE(distX,distY,e);
		return;
	}
	tempE = *e;
	moveE(distX,distY,&tempE);
	uint8_t valid=0x00;
	if(isValid(tempE.x+entDat[tempE.type].hitBox.x, tempE.y+entDat[tempE.type].hitBox.y))valid|=_BV(0);//top left corner is in unwalkable tile
	if(isValid(tempE.x+entDat[tempE.type].hitBox.x+entDat[tempE.type].hitBox.w, tempE.y+entDat[tempE.type].hitBox.y))valid|=_BV(1);//top right corner is in unwalkable tile
	if(isValid(tempE.x+entDat[tempE.type].hitBox.x, tempE.y+entDat[tempE.type].hitBox.y+entDat[tempE.type].hitBox.h))valid|=_BV(2);//bottom left corner is in unwalkable tile
	if(isValid(tempE.x+entDat[tempE.type].hitBox.x+entDat[tempE.type].hitBox.w, tempE.y+entDat[tempE.type].hitBox.y+entDat[tempE.type].hitBox.h))valid|=_BV(3);//bottom right corner is in unwalkable tile

	switch(valid){
		case 0x0F: //all good
			break;
		case 0x01:
		case 0x02:
		case 0x04:
		case 0x08://in corner stop all movement
			distX=0;distY=0;
			break;
		case 0x0C:
		case 0x03://in top/bottom stop all y movement
			distY=0;
			break;
		case 0x0A:
		case 0x05://in sides stop all x movement
			distX=0;
			break;
		case 0x0E://in outside corner top left
			if(isValid(tempE.x+entDat[tempE.type].hitBox.x-distX, tempE.y+entDat[tempE.type].hitBox.y)){
				distX=0;
			}else{
				distY=0;
			}
			break;
		case 0x0D://in outside corner top right
			if(isValid(tempE.x+entDat[tempE.type].hitBox.x+entDat[tempE.type].hitBox.w-distX, tempE.y+entDat[tempE.type].hitBox.y)){
				distX=0;
			}else{
				distY=0;
			}
			break;
		case 0x0B://in outside corner bottom left
			if(isValid(tempE.x+entDat[tempE.type].hitBox.x-distX, tempE.y+entDat[tempE.type].hitBox.y+entDat[tempE.type].hitBox.h)){
				distX=0;
			}else{
				distY=0;
			}
			break;
		case 0x07://in outside corner bottom right
			if(isValid(tempE.x+entDat[tempE.type].hitBox.x+entDat[tempE.type].hitBox.w-distX, tempE.y+entDat[tempE.type].hitBox.y+entDat[tempE.type].hitBox.h)){
				distX=0;
			}else{
				distY=0;
			}
			break;
		case 0x00://not sure how possible (stuck)
			fprintf(debuglog, "stuck in wall\n");
			distX=0;distY=0;
			break;
		case 0x09:
		case 0x06:
			distX=0;distY=0;
			break;
		default:
			fprintf(debuglog, "bad valid state on collision\n");
			distX=0;distY=0;
			break;
	}
	//finaly do the real move
	moveE(distX,distY,e);
}

tile* getTile(float x, float y){
	return &map[(int)(x)/TILE_SIZE][(int)(y)/TILE_SIZE];
}

int isValid(float x, float y){
	if(x<0)return 0;
	if(y<0)return 0;
	if(x>MAP_WIDTH*TILE_SIZE)return 0;
	if(y>MAP_HEIGHT*TILE_SIZE)return 0;
	if(map[(int)(x)/TILE_SIZE][(int)(y)/TILE_SIZE].flags&_BV(0)){
		return 0;
	}else{
		return 1;
	}
}

void onRender(){
	if(mode==1){
		//prepare text
		sprintf(text, "%d HP" ,(Entities[playerID].health));
		sprintf(text2, "Level %d" ,level);
		//Clear screen
		SDL_RenderClear( gRenderer );
		
		//draw stuff
		//draw floor
		int i,j,k;
		for(i=0;i<MAP_HEIGHT;i++){
			for(j=0;j<MAP_WIDTH;j++){
				SDLP_drawFromSheet(&floorS, map[j][i].floorID, (SDL_Point){TILE_SIZE*j,TILE_SIZE*i});
			}
		}
		//draw objects+entitiys
		for(i=0,k=0;i<MAP_HEIGHT;i++){
			while((Entities[k].y<i*TILE_SIZE)&&k<entityC){
				switch(Entities[k].type){
				case 0://player
					SDLP_drawFromSheet(&playerS, (4*Entities[k].facing)+(Entities[k].frame), (SDL_Point){Entities[k].x-entDat[Entities[k].type].xOff, Entities[k].y-entDat[Entities[k].type].yOff});
					break;
				case 1://ghost
					SDLP_drawFromSheet(&mobS, (3*Entities[k].facing)+((gameTime/500)%3), (SDL_Point){Entities[k].x-entDat[Entities[k].type].xOff, Entities[k].y-entDat[Entities[k].type].yOff});
					break;
				case 2://bat
					SDLP_drawFromSheet(&mobS, (3*Entities[k].facing)+((gameTime/100)%3)+12, (SDL_Point){Entities[k].x-entDat[Entities[k].type].xOff, Entities[k].y-entDat[Entities[k].type].yOff});
					break;
				case 3://bullet
				case 4://fabric
				case 5://bomb
				case 6://wet fabric
				case 7://gem
				case 8://key
					SDLP_drawFromSheet(&itemS, Entities[k].type-3, (SDL_Point){Entities[k].x-entDat[Entities[k].type].xOff, Entities[k].y-entDat[Entities[k].type].yOff});
					break;
				default:
					break;
			}
				k++;
			}
			for(j=0;j<MAP_WIDTH;j++){
				if(map[j][i].objectID){
					SDLP_drawFromSheet(&objectS, map[j][i].objectID, (SDL_Point){TILE_SIZE*j,TILE_SIZE*(i-2)});
				}
			}
		}
		
		//draw hud
		tempRectC = (SDL_Rect){0,0,79,52};
		SDL_RenderCopy(gRenderer, hud, &tempRectC, &tempRectC);
		//draw text
		SDLP_drawStringScaledColor(&testFont, text, (SDL_Point){0,2},2,BLOOD_RED);
		SDLP_drawStringScaledColor(&testFont, text2, (SDL_Point){SCREEN_WIDTH-100,2},2,SKY_BLUE);
		//draw inventory
		if(inventory>=4)
			SDLP_drawFromSheet(&itemS, inventory-3, SDL_PointAdd((SDL_Point){0,20},camPos));
		
		//Update screen
		SDL_RenderPresent( gRenderer );
	}else if(mode==0){
		//prepare text
		if(submode==2){
			sprintf(text, "You have completed\n       all the levels\n  and won the game!");
		}else if(submode==1){
			sprintf(text, "Game over,\ntry again");
		}else{//assume submode=0
			sprintf(text, " JAM-\nCRAWL");
		}
		sprintf(text2, "Press SPACE to start a new game");
		sprintf(text3, "Controls: WASD to move,\nSPACE to attack/use item,\nDELETE to restart level.");
		
		//Clear screen
		SDL_RenderClear( gRenderer );
		
		//render background/splash
		SDL_RenderCopy(gRenderer, splash, NULL,NULL);
		
		//render text lines
		if(submode==2){
			SDLP_drawStringScaledColor(&testFont, text, (SDL_Point){(SCREEN_WIDTH/2)-225, 20},4,GOLDEN);
		}else if(submode==1){
			SDLP_drawStringScaledColor(&testFont, text, (SDL_Point){(SCREEN_WIDTH/2)-120, 20},4,GOLDEN);
		}else{//assume submode=0
			SDLP_drawStringScaledColor(&testFont, text, (SDL_Point){(SCREEN_WIDTH/2)-210, 20},12,GOLDEN);
		}
		SDLP_drawStringScaledColor(&testFont, text2, (SDL_Point){(SCREEN_WIDTH/2)-190, (3*SCREEN_HEIGHT)/4},2,SKY_BLUE);
		SDLP_drawStringScaledColor(&testFont, text3, (SDL_Point){20, SCREEN_HEIGHT-54},2,SKY_BLUE);
		
		//Update screen
		SDL_RenderPresent( gRenderer );
	}
}

void onClose(){
	SDL_DestroyRenderer( gRenderer );
	gRenderer = NULL;
	SDL_DestroyWindow(window);
	window = NULL;
	IMG_Quit();
	SDL_Quit();
}

void reset(int lv){
	time3=clock();
	int i,j;
	switch(lv){
		case 4:
			Entities[0]=(entity){320,680,0,FACE_UP,0,0,100,0,0,100,25};//Entities[playerID] is the player
			Entities[1]=(entity){160,240,2,FACE_DOWN,0,_BV(1),70,150,250,100,10};
			Entities[2]=(entity){480,240,2,FACE_DOWN,0,_BV(1),70,150,250,100,10};
			Entities[3]=(entity){160,440,2,FACE_DOWN,0,_BV(1),70,150,250,100,10};
			Entities[4]=(entity){480,440,2,FACE_DOWN,0,_BV(1),70,150,250,100,10};
			Entities[5]=(entity){160,640,2,FACE_DOWN,0,_BV(1),70,150,250,100,10};
			Entities[6]=(entity){480,640,2,FACE_DOWN,0,_BV(1),70,150,250,100,10};
			Entities[7]=(entity){320,67,7,0,0,0,0,0,0,0,0};
			Entities[8]=(entity){60,200,8,0,0,0,0,0,0,0,0};
			Entities[9]=(entity){580,200,8,0,0,0,0,0,0,0,0};
			Entities[10]=(entity){60,400,8,0,0,0,0,0,0,0,0};
			Entities[11]=(entity){580,400,8,0,0,0,0,0,0,0,0};
			Entities[12]=(entity){60,600,8,0,0,0,0,0,0,0,0};
			Entities[13]=(entity){580,600,8,0,0,0,0,0,0,0,0};
			playerID=0;
			entityC = 14;
			level=4;
			for(i=0;i<MAP_WIDTH;i++){
				for(j=0;j<MAP_HEIGHT;j++){
					map[i][j]=mapD[j][i];
				}
			}
			break;
		case 3:
			Entities[0]=(entity){180,360,0,FACE_DOWN,0,0,100,0,0,100,25};//Entities[playerID] is the player
			Entities[1]=(entity){60,320,1,FACE_UP,0,_BV(1),25,150,100,50,10};
			Entities[2]=(entity){60,320,1,FACE_UP,0,0,25,150,100,50000,3};
			Entities[3]=(entity){60,320,1,FACE_UP,0,0,25,150,100,50000,3};
			Entities[4]=(entity){60,320,1,FACE_UP,0,0,25,150,100,50000,3};
			Entities[5]=(entity){60,400,2,FACE_RIGHT,0,_BV(1),40,150,250,50,10};
			Entities[6]=(entity){300,440,2,FACE_RIGHT,0,_BV(1),40,150,250,50,10};
			Entities[7]=(entity){300,280,2,FACE_RIGHT,0,0,20,150,250,50000,3};//decoy and pester
			Entities[8]=(entity){280,67,7,0,0,0,0,0,0,0,0};
			Entities[9]=(entity){60,200,8,0,0,0,0,0,0,0,0};
			playerID=0;
			entityC = 10;
			level=3;
			for(i=0;i<MAP_WIDTH;i++){
				for(j=0;j<MAP_HEIGHT;j++){
					map[i][j]=mapC[j][i];
				}
			}
			break;
		case 2:
			Entities[0]=(entity){600,720,0,FACE_UP,0,0,100,0,0,100,25};//Entities[playerID] is the player
			Entities[1]=(entity){540,560,1,FACE_UP,0,0,25,540,500,50,10};
			Entities[2]=(entity){660,560,1,FACE_UP,0,0,25,660,500,50,10};
			Entities[3]=(entity){380,640,1,FACE_LEFT,0,0,25,300,600,50,10};
			Entities[4]=(entity){820,640,1,FACE_RIGHT,0,_BV(1),25,900,600,50,10};
			Entities[5]=(entity){260,560,2,FACE_RIGHT,0,_BV(1),50,150,250,50,10};
			Entities[6]=(entity){940,560,2,FACE_RIGHT,0,0,50,150,250,50,10};
			Entities[7]=(entity){180,680,2,FACE_RIGHT,0,0,50,150,250,50,10};
			Entities[8]=(entity){1020,680,2,FACE_RIGHT,0,_BV(1),50,150,250,50,10};
			Entities[9]=(entity){460,120,2,FACE_DOWN,0,0,75,150,250,50,10};
			Entities[10]=(entity){740,120,2,FACE_DOWN,0,0,75,150,250,50,10};
			Entities[11]=(entity){1020,480,8,0,0,0,0,0,0,0,0};
			Entities[12]=(entity){600,207,7,0,0,0,0,0,0,0,0};
			playerID=0;
			entityC = 13;
			level=2;
			for(i=0;i<MAP_WIDTH;i++){
				for(j=0;j<MAP_HEIGHT;j++){
					map[i][j]=mapB[j][i];
				}
			}
			break;
		case 0:
		case 1:
		default:
			Entities[0]=(entity){100,100,0,FACE_DOWN,0,0,100,0,0,100,25};//Entities[playerID] is the player
			Entities[1]=(entity){140,170,1,FACE_UP,0,_BV(1),25,150,100,50,10};
			Entities[2]=(entity){420,220,2,FACE_RIGHT,0,_BV(1),40,150,250,50,10};
			Entities[3]=(entity){420,67,7,0,0,0,0,0,0,0,0};
			Entities[4]=(entity){60,240,8,0,0,0,0,0,0,0,0};
			playerID=0;
			entityC = 5;
			level=1;
			for(i=0;i<MAP_WIDTH;i++){
				for(j=0;j<MAP_HEIGHT;j++){
					map[i][j]=mapA[j][i];
				}
			}
	}
	inventory=0;
}

void sortEnt(){//sort the list of entities by assending y using insertion sort
	entity tempEnt;
	int i,j;
	for(i=1;i<entityC;i++){
		j=i;
		while((j>0)&&(Entities[j-1].y>Entities[j].y)){
			tempEnt=Entities[j];
			Entities[j]=Entities[j-1];
			Entities[j-1]=tempEnt;
			j--;
		}
	}
	for(i=0;1;i++){
		if(Entities[i].type==0){
			playerID=i;
			break;
		}
	}
}