#ifndef SDL_UTILS_H
#define SDL_UTILS_H

#include <math.h>

//=========================================================================================
//define macros
//=========================================================================================
#define _BV(bit) (1 << (bit))


//=========================================================================================
//typedefs
//=========================================================================================
typedef struct {
	SDL_Texture* tex;
	uint16_t dx,dy,rows,cols;
} spriteSheet;

typedef struct {
	SDL_Texture* tex;
	unsigned char start,end;
	uint16_t lineHeight,spaceLen;
	SDL_Rect* meta;
} bitFont;


//=========================================================================================
//Global variables
//=========================================================================================
FILE* debuglog;//the debuglog file pointer (open in init)
SDL_Event Event;//event used in event loop
SDL_Window* window = NULL;//main window of the game
SDL_Surface* screenSurface = NULL;//surface of the window (almost redundant due to renderer)
SDL_Renderer* gRenderer = NULL;//2d hardware accelerator
SDL_Surface* loadSurf = NULL;//temp surface used when loading assets

SDL_Point camPos={0,0};
//temp variables (used to minimize temp variable allocation)
SDL_Rect tempRectA, tempRectB;//used in most render functions for source/target rectangles
int tempIntA;//used in loops as temp variables
SDL_Point tempPointA;//used in multidraw functions

//=========================================================================================
//Function heads
//=========================================================================================
//loading
SDL_Texture* SDLP_loadTex(const char* filename);
int SDLP_loadSheet(const char* filename, spriteSheet* sheet, const uint16_t width, const uint16_t hight);
int SDLP_loadFont(const char* filename, bitFont* font, const unsigned char start, const unsigned char end, const uint16_t lineHeight, const uint16_t spaceLen, SDL_Rect* meta);
//draw
void SDLP_drawFromSheet(spriteSheet* sheet, const int index, const SDL_Point where);
void SDLP_drawFromSheetScaled(spriteSheet* sheet, const int index, const SDL_Point where, const float scale);
void SDLP_drawFromSheetScaleXY(spriteSheet* sheet, const int index, const SDL_Rect where);
//draw(text)
void SDLP_drawChar(bitFont* font, const unsigned char C, const SDL_Point where);
void SDLP_drawCharScaled(bitFont* font, const unsigned char C, const SDL_Point where, const float scale);
int SDLP_drawString(bitFont* font, const unsigned char* C, const SDL_Point where);
int SDLP_drawStringScaled(bitFont* font, const unsigned char* C, const SDL_Point where, const float scale);
//math
float distF(float x1,float y1, float x2, float y2);
//=========================================================================================
//Functions
//=========================================================================================
//math
inline SDL_Point SDL_PointAdd(const SDL_Point A, const SDL_Point B){
	return (SDL_Point){A.x+B.x,A.y+B.y};
}
inline SDL_Point SDL_PointSub(const SDL_Point A, const SDL_Point B){
	return (SDL_Point){A.x-B.x,A.y-B.y};
}
inline SDL_Point SDL_PointNeg(const SDL_Point A){
	return (SDL_Point){-A.x,-A.y};
}

//loading
SDL_Texture* SDLP_loadTex(const char* filename){
	SDL_Texture* tex;
	loadSurf = IMG_Load(filename);
	if(loadSurf == NULL){
		fprintf(debuglog, "Unable to load image %s !\n", filename);
		return NULL;
	}
	tex = SDL_CreateTextureFromSurface( gRenderer, loadSurf );
	SDL_FreeSurface(loadSurf);
	loadSurf = NULL;
	return tex;
}

int SDLP_loadSheet(const char* filename, spriteSheet* sheet, const uint16_t width, const uint16_t hight){
	loadSurf = IMG_Load(filename);
	if(loadSurf == NULL){
		fprintf(debuglog, "Unable to load image %s !\n", filename);
		return -1;
	}
	sheet->tex = SDL_CreateTextureFromSurface( gRenderer, loadSurf );
	sheet->dx=width;
	sheet->dy=hight;
	sheet->rows=loadSurf->h/hight;
	sheet->cols=loadSurf->w/width;
	SDL_FreeSurface(loadSurf);
	loadSurf = NULL;
	return 1;
}

int SDLP_loadFont(const char* filename, bitFont* font, const unsigned char start, const unsigned char end, const uint16_t lineHeight, const uint16_t spaceLen, SDL_Rect* meta){
	font->start = start;
	font->end = end;
	font->lineHeight = lineHeight;
	font->spaceLen = spaceLen;
	font->meta = meta;
	font->tex = SDLP_loadTex(filename);
	if(font->tex == NULL)return -1;
	return 1;
}

//draw
void SDLP_drawFromSheet(spriteSheet* sheet, const int index, SDL_Point where){
	if(index>=((sheet->rows)*(sheet->cols)))return;
	where = SDL_PointSub(where, camPos);
	tempRectA.x=(sheet->dx)*(index%(sheet->cols));
	tempRectA.y=(sheet->dy)*(index/(sheet->cols));
	tempRectA.w=sheet->dx;
	tempRectA.h=sheet->dy;
	
	tempRectB.x=where.x;
	tempRectB.y=where.y;
	tempRectB.w=sheet->dx;
	tempRectB.h=sheet->dy;
	
	SDL_RenderCopy(gRenderer, sheet->tex, &tempRectA, &tempRectB);
}

void SDLP_drawFromSheetScaled(spriteSheet* sheet, const int index, SDL_Point where, const float scale){
	if(index>=((sheet->cols)*(sheet->cols)))return;
	where = SDL_PointSub(where, camPos);
	tempRectA.x=(sheet->dx)*(index%(sheet->cols));
	tempRectA.y=(sheet->dy)*(index/(sheet->cols));
	tempRectA.w=sheet->dx;
	tempRectA.h=sheet->dy;
	
	tempRectB.x=where.x;
	tempRectB.y=where.y;
	tempRectB.w=scale*(sheet->dx);
	tempRectB.h=scale*(sheet->dy);
	
	SDL_RenderCopy(gRenderer, sheet->tex, &tempRectA, &tempRectB);
}

void SDLP_drawFromSheetScaleXY(spriteSheet* sheet, const int index, SDL_Rect where){
	if(index>=((sheet->cols)*(sheet->cols)))return;
	where = (SDL_Rect){where.x-camPos.x,where.y-camPos.y,where.w,where.h};
	tempRectA.x=(sheet->dx)*(index%(sheet->cols));
	tempRectA.y=(sheet->dy)*(index/(sheet->cols));
	tempRectA.w=sheet->dx;
	tempRectA.h=sheet->dy;
	tempRectB=where;
	
	SDL_RenderCopy(gRenderer, sheet->tex, &tempRectA, &tempRectB);
}

//draw(text)
void SDLP_drawChar(bitFont* font, const unsigned char C, SDL_Point where){
	if((C<font->start)||(C>font->end))return;
	//where = SDL_PointSub(where, camPos);
	tempRectB = (SDL_Rect){where.x, where.y, font->meta[C-font->start].w, font->meta[C-font->start].h};
	SDL_RenderCopy(gRenderer, font->tex, &(font->meta[C-font->start]), &tempRectB);
}

void SDLP_drawCharScaled(bitFont* font, const unsigned char C, SDL_Point where, const float scale){
	if((C<font->start)||(C>font->end))return;
	//where = SDL_PointSub(where, camPos);
	tempRectB = (SDL_Rect){where.x, where.y, scale*font->meta[C-font->start].w, scale*font->meta[C-font->start].h};
	SDL_RenderCopy(gRenderer, font->tex, &(font->meta[C-font->start]), &tempRectB);
}
int SDLP_drawStringColor(bitFont* font, const unsigned char* C, SDL_Point where, int r, int g, int b){
	SDL_SetTextureColorMod(font->tex, r,g,b);
	SDLP_drawString(font, C, where);
	SDL_SetTextureColorMod(font->tex, 0xFF,0xFF,0xFF);
}
int SDLP_drawString(bitFont* font, const unsigned char* C, SDL_Point where){
	tempIntA=0;
	tempPointA=where;
	while(C[tempIntA]!=0){
		if(C[tempIntA] == '\n'){
			tempPointA.x=where.x;
			tempPointA.y+=font->lineHeight;
		}else{
			if(C[tempIntA] == ' '){
				tempPointA.x+=font->spaceLen;
			}else if((C[tempIntA]<font->start)||(C[tempIntA]>font->end)){
				tempPointA.x+=font->spaceLen;
			}else{
				SDLP_drawChar(font, C[tempIntA], tempPointA);
				tempPointA.x+=font->meta[C[tempIntA]-font->start].w;
			}
		}
		tempIntA++;
	}
	return tempIntA;
}
int SDLP_drawStringScaledColor(bitFont* font, const unsigned char* C, SDL_Point where, const float scale, int r, int g, int b){
	SDL_SetTextureColorMod(font->tex, r,g,b);
	SDLP_drawStringScaled(font, C, where, scale);
	SDL_SetTextureColorMod(font->tex, 0xFF,0xFF,0xFF);
}
int SDLP_drawStringScaled(bitFont* font, const unsigned char* C, SDL_Point where, const float scale){
	tempIntA=0;
	tempPointA=where;
	while(C[tempIntA]!=0){
		if(C[tempIntA] == '\n'){
			tempPointA.x=where.x;
			tempPointA.y+=scale*font->lineHeight;
		}else{
			if(C[tempIntA] == ' '){
				tempPointA.x+=scale*font->spaceLen;
			}else if((C[tempIntA]<font->start)||(C[tempIntA]>font->end)){
				tempPointA.x+=scale*font->spaceLen;
			}else{
				SDLP_drawCharScaled(font, C[tempIntA], tempPointA, scale);
				tempPointA.x+=scale*font->meta[C[tempIntA]-font->start].w;
			}
		}
		tempIntA++;
	}
	return tempIntA;
}

float distF(float x1,float y1, float x2, float y2){
	return sqrt(((x1-x2)*(x1-x2))+((y1-y2)*(y1-y2)));
}

SDL_Rect rectAdd(SDL_Rect R, SDL_Point P){
	return (SDL_Rect){R.x+P.x,R.y+P.y,R.w,R.h};
}
SDL_Rect rectSub(SDL_Rect R, SDL_Point P){
	return (SDL_Rect){R.x-P.x,R.y-P.y,R.w,R.h};
}

#endif